#
#    Copyright (C) 2003-2004  James Tauber
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

import urllib

import templates.editform as template
from providers.page import HTMLPage, ForbiddenPage
from web_resource import PagePart


def edit_menu_part(request, resource, config):

    if request.logged_in() and not resource.is_read_only():
        edit_page_href = config.edit_href + "?" + config.edit_resource_field + "=" + urllib.quote(resource.get_id())
        menu_part = template.edit_menu % locals()
    else:
        menu_part = ""
        
    return menu_part



class EditFormProvider:
    
    
    def __init__(self, resource_manager, lfs, config):

        self.resource_manager = resource_manager
        self.lfs              = lfs
        self.config           = config

        self.edit_path          = config.edit_path
        self.resource_name      = config.edit_resource_field

       
    def get(self, resource_id, request, main_resource):

        if resource_id != self.edit_path:
            return None

        if not request.logged_in():
            return ForbiddenPage(resource_id, self.resource_manager, request, self.config)

        resource_to_edit = request.fields.getvalue(self.resource_name)

        outer_page_part = EditFormPagePart(resource_id, resource_to_edit, self.resource_manager, request, self.config)
        
        html_page = HTMLPage(resource_id, outer_page_part, self.resource_manager, request, self.config)
        html_page.expiry = 0
        
        return html_page
        

            
class EditFormPagePart(PagePart):


    def __init__(self, resource_id, resource_to_edit, resource_manager, request, config):
        post_action = config.put_href
        page_content_name = config.put_content_field
        page_content_type_name = config.put_content_type_field
        resource_name = config.put_resource_field
        cancel_action = urllib.quote(resource_to_edit)

        if resource_to_edit:
            page = resource_manager.get(resource_to_edit, request, self)
    
            if page.is_read_only():
                self.title = template.edit_read_only_title % locals()
                self.html = template.edit_read_only_title % locals()
            
            else:
        
                time = None
                site_title = config.site_title
        
                if page.exists():
                    page_title = page.get_title()
                    title = template.page_title % locals()
                    page_content = page.get_content()
                    page_content_type = page.get_content_type()
                    time = page.get_lastmod()
                else: # new wiki word
                    title = template.new_page_title % locals()
                    page_content = template.new_page_content % locals()
                    page_content_type = config.editform_default_type
                
                content = template.edit_form % locals()
        
                self.title = title
                self.html  = content
        else:
            self.title = template.edit_missing_id_title % locals()
            self.html = template.edit_missing_id_content % locals()            
