#
#    Copyright (C) 2003-2004  James Tauber
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

import re

from web_resource import TopLevelResource, FileObjectResource



class EnclosureProvider:
    
    
    def __init__(self, lfs, config):
        self.lfs    = lfs
        self.config = config
        
        
    def get(self, resource_id, request, main_resource):
        match = re.match("(.*)/__(\w+)__(\d+)", resource_id)
        if match:
            base = match.group(1)
            enctype = match.group(2)
            index = match.group(3)
            
            enclosure = self.lfs.get(base).enclosure(enctype, index)
            
            if enclosure.exists():
                mime_type = "image/png" # @@@
                return EnclosureResource(resource_id, mime_type, enclosure)
            else:
                return Enclosure404(resource_id)
        else:
            return None



class EnclosureResource(FileObjectResource, TopLevelResource):


    def __init__(self, resource_id, mime_type, enclosure):
        TopLevelResource.__init__(self, resource_id, mime_type)
        FileObjectResource.__init__(self, enclosure)



class Enclosure404(TopLevelResource):


    def __init__(self, resource_id):
        TopLevelResource.__init__(self, resource_id, "text/plain")
        self.status = "404 Not Found"
        self.content = "Enclosure not found"
        
        
    def exists(self):
        return False
        
        
    def get_content(self):
        self.content