#
#    Copyright (C) 2003-2004  James Tauber
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

import urllib

from auth import OK
from providers.page import HTMLPage
import templates.login_logout as template
from web_resource import PagePart


def menu_part(request, resource, config):
    if request.logged_in():
        user_id = request.get_user()
        logout_href = config.logout_href + "?" + config.log_return_field + "=" + urllib.quote(request.relative_path)
        menu_part = template.logged_in_menu % locals()
    else:
        login_href = config.login_href + "?" + config.log_return_field + "=" + urllib.quote(request.relative_path)
        menu_part = template.logged_out_menu % locals()

    return menu_part



class LoginLogoutProvider:
    
    
    def __init__(self, resource_manager, lfs, config):

        self.resource_manager = resource_manager
        self.lfs              = lfs
        self.config           = config
        
        self.login_path  = config.login_path
        self.logout_path = config.logout_path



    def get(self, resource_id, request, main_resource):
        if resource_id == self.login_path:
            page_part = LoginPagePart(resource_id, request, self.config)
        elif resource_id == self.logout_path:
            page_part = LogoutPagePart(resource_id, request, self.config)
        else:
            return None
        
        html_page = HTMLPage(resource_id, page_part, self.resource_manager, request, self.config)
        html_page.expiry = 0

        # @@@ put on HTMLPage itself
        if hasattr(page_part, "cookie"):
            html_page.cookie = page_part.cookie

        return html_page
        


class LoginPagePart(PagePart):


    def __init__(self, resource_id, request, config):
        PagePart.__init__(self, resource_id)

        # form fields
        email_name    = "email"
        password_name = "passwd"
        return_name   = config.log_return_field
        
        session_status = request.check_session()
        action = config.login_href

        return_path = request.fields.getvalue(return_name, "")

        if request.method == "GET":
            status_code = session_status[0]
            if session_status[0] == OK:
                user_id = session_status[1]
                content = template.already_logged_in % locals()
            else:
                content = template.login_form % locals()
            
        else: # POST
            # log out if someone else
            session_status = request.check_session()
            if session_status[0] == OK:
                request.delete_session(session_status[2])
            
            email  = request.fields.getvalue(email_name, "")
            passwd = request.fields.getvalue(password_name, "")
            return_path = request.fields.getvalue(return_name, "")
            return_href = config.cgi_root + return_path
            
            status_code = session_status[0]
            
            if request.correct_password(email, passwd):

                # @@@ should consider putting cookie creation code on
                # response object
                import Cookie
                ip_address = request.remote_address
                self.cookie = Cookie.SimpleCookie()
                self.cookie["session_id"] = request.create_session(email, ip_address)
                self.cookie["session_id"]["expires"] = 86400 # @@@ parameterize out
                self.cookie["session_id"]["path"] = "/" # @@@ should be home_href
                
                # bit of a hack, inserting the cookie after the fact
                request.cookie = self.cookie
                
                content = template.successful_login % locals()
            else:
                content = template.incorrect_password % locals()

        self.title = template.login_title
        self.html = content
        
        

class LogoutPagePart(PagePart):


    def __init__(self, resource_id, request, config):
        PagePart.__init__(self, resource_id)
        
        return_name = config.log_return_field
        action = config.logout_href

        return_path = request.fields.getvalue(return_name, "")
        return_href = config.home_href + return_path

        if request.method == "GET":
            content = template.confirm_logout % locals()
        else: # POST
            session_status = request.check_session()
            if session_status[0] == OK:
                request.delete_session(session_status[2])
                user_id = session_status[1]
                content = template.successful_logout % locals()
            else:
                status_code = session_status[0]
                content = template.not_logged_in % locals()

        self.title = template.logout_title
        self.html = content        
