#
#    Copyright (C) 2003-2004  James Tauber
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

import templates.page as template
from web_resource import PagePart


class MenuProvider:


    def __init__(self, resource_manager, file_system, config):
    
        self.resource_manager = resource_manager
        self.file_system      = file_system
        self.config           = config
        
        self.menu_key = config.menu_key

        self.menu_providers = []

        
    def register_provider(self, part_provider):
        self.menu_providers.append(part_provider)


    def get(self, resource_id, request, main_resource):
        if resource_id == self.menu_key:
            return MenuPagePart(self.resource_manager, self.menu_providers, request, main_resource, self.config)
        else:
            return None



class MenuPagePart(PagePart):
    """
    The page part representing the full menu, including the content provided by
    menu providers.
    
    This resource is never requested directly, only as part of a page.
    
    The main (or inner) menu, which forms part of the content of this resource,
    is a resource in its own right with id config.main_menu_key. The main menu
    can be requested directly and edited like a normal page part resource.
    """


    def __init__(self, resource_manager, menu_providers, request, main_resource, config):
        self.main_menu      = resource_manager.get(config.main_menu_key, request, self)
        self.menu_providers = menu_providers
        self.request        = request
        self.main_resource  = main_resource
        self.config         = config

        
    def get_html(self):
        html = self.main_menu.get_html()
        html += template.menu_separator % locals()
        for provider in self.menu_providers:
            html += provider(self.request, self.main_resource, self.config)
        
        return html
