#
#    Copyright (C) 2003-2004  James Tauber
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""
POSTing to the 'put' resource is intended as an alternative to PUTing a
resource in cases like HTML forms where you can't directly PUT.
"""

import urllib

import templates.put as template
from providers.page import HTMLPage, ForbiddenPage
from web_resource import PagePart


class PutProvider:
    
    
    def __init__(self, resource_manager, lfs, config):

        self.resource_manager = resource_manager
        self.lfs              = lfs
        self.config           = config

        self.put_path       = config.put_path

       
    def get(self, resource_id, request, main_resource):

        if resource_id != self.put_path:
            return None

        if request.method == "GET":
            return ForbiddenPage(resource_id,  self.resource_manager, request, self.config)

        if not request.logged_in():
            return ForbiddenPage(resource_id,  self.resource_manager, request, self.config)

        page_part = PutPagePart(resource_id, self.resource_manager, request, self.config)

        return HTMLPage(resource_id, page_part, self.resource_manager, request, self.config)
            


class PutPagePart(PagePart):

    def __init__(self, resource_id, resource_manager, request, config):
        PagePart.__init__(self, resource_id)

        resource_to_put = request.fields.getvalue(config.put_resource_field)
        
        if resource_to_put:
           
            page = resource_manager.get(resource_to_put, request, self)

            if page.is_read_only():
                title = template.put_read_only_title % locals()
                content = template.put_read_only_content % locals()
    
            else:
                
                new_content =  request.fields.getvalue(config.put_content_field)
                new_content_type = request.fields.getvalue(config.put_content_type_field)
                    
                if page.exists():
                    replace = True
                else:
                    replace = False
                    
                page.set_content(new_content, new_content_type)
                
                resource_link = config.cgi_root + urllib.quote(resource_to_put)
                resource_label = resource_to_put
                resource_type = new_content_type
                
                if replace:
                    title = template.put_replace_title % locals()
                    content = template.put_replace_content % locals()
                else:
                    title = template.put_create_title % locals()
                    content = template.put_create_content % locals()

        else: # no resource_id provided
        
            title = template.put_missing_id_title % locals()
            content = template.put_missing_id_content % locals()            

        self.title = title
        self.html  = content
